/*
 *
 * Copyright (C) 2003 Sadysta ( http://sadysta.ath.cx )
 * Copyright (C) 2007 Tomasz Ostrowski ( http://ostry.w.szu.pl ) - modified from 1.8.9 source
 *
 * This file is part of Free GG.
 *
 * Free GG is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Free GG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Free GG; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Main.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
#include "GaduThread.h"
#include "Contact.h"
#include "Talk.h"
#include "Settings.h"
#include "Search.h"
#include "SMS.h"

#include "NewAccnt.h"
#include "DelAccnt.h"
#include "ChPwd.h"
#include "RemindPwd.h"

#include "StatusDesc.h"
#include "History.h"
#include "Me.h"
#include "Activity.h"

#include "PluginManager.h"

//---------------------------------------------------------------------------
#pragma link "trayicon"
#pragma resource "*.dfm"
TMainForm *MainForm;
extern TGaduThread *gg_thread;
extern PluginManager *PlugManager;

//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
        : TForm(Owner)
{
    this->Caption = "Free GG " + GetFileVersion(Application->ExeName);
    PlugManager = new PluginManager((ExtractFileDir(Application->ExeName)+"\\plugins\\").c_str());
    bHistoryIsOpen = false;
    bCleanAtExit = false;
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
    delete PlugManager;
    PlugManager = NULL;
}

//---------------------------------------------------------------------------
/** Get Executable version */
AnsiString __fastcall TMainForm::GetFileVersion(AnsiString FileName)
{
	AnsiString asVer="";
	VS_FIXEDFILEINFO *pVsInfo;
	unsigned int iFileInfoSize = sizeof( VS_FIXEDFILEINFO );

	int iVerInfoSize = GetFileVersionInfoSize(FileName.c_str(), NULL);
    if (iVerInfoSize!= 0)
    {
		char *pBuf = new char[iVerInfoSize];
        if (GetFileVersionInfo(FileName.c_str(),0, iVerInfoSize, pBuf ) )
        {
            if (VerQueryValue(pBuf, "\\",(void **)&pVsInfo,&iFileInfoSize))
            {
				asVer  = IntToStr( HIWORD(pVsInfo->dwFileVersionMS) )+".";
				asVer += IntToStr( LOWORD(pVsInfo->dwFileVersionMS) )+".";
				asVer += IntToStr( HIWORD(pVsInfo->dwFileVersionLS) )+".";
				asVer += IntToStr( LOWORD(pVsInfo->dwFileVersionLS) );
            }
        }
        delete [] pBuf;
    }
    return asVer;
}

/** Call SetLayeredWindowAttributes if available
	(>= win2k)
*/
void TMainForm::SetTransparentWindow(int opacity)
{
	typedef DWORD (WINAPI *FWINLAYER)(HWND hwnd,DWORD crKey,BYTE bAlpha,DWORD dwFlags);
	HINSTANCE hDll = LoadLibrary("user32.dll");
	FWINLAYER setLayeredWindowAttributes = (FWINLAYER)GetProcAddress(hDll,"SetLayeredWindowAttributes");

	if(setLayeredWindowAttributes != NULL)
	{
		SetWindowLong(this->Handle, GWL_EXSTYLE, WS_EX_LAYERED);
		setLayeredWindowAttributes(this->Handle, 0, 255-opacity, 2);
	}
}

void __fastcall TMainForm::status3Click(TObject *Sender)
{
    Log->Lines->Add ("Zmiana statusu: " + IntToStr (((TMenuItem*) Sender) -> Tag));
    ((TMenuItem*)Sender) -> Checked = true;
    TrayIcon1 -> IconIndex = ((TMenuItem*)Sender) -> ImageIndex;
    if (!gg_thread -> dead) gg_thread -> ChangeStatus (((TMenuItem*)Sender) -> Tag, StatusDescForm -> Edit1 -> Text, fonly -> Checked);
    if (((TMenuItem*)Sender) -> Tag == 4) StatusBar -> Panels -> Items[0] -> Text = "Rozlaczony";
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Dodaj1Click(TObject *Sender)
{
    for (int i = 0; i < ContactForm -> ComponentCount; i++)
        if (ContactForm -> Components[i] -> Tag == 101) ((TEdit*)ContactForm -> Components[i]) -> Text = "";
        else
            if (ContactForm -> Components[i] -> Tag == 102) ((TComboBox*)ContactForm -> Components[i]) -> ItemIndex = 0;
    ContactForm -> ShowModal();
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::ListDblClick(TObject *Sender)
{
    TTalkForm *T;
    bool exists = false;

    if (List -> Selected)
        if (StrToIntDef(List -> Selected -> SubItems -> Strings[0], -1) != -1)
        {
            for (int i = 0; i < this->ComponentCount; i++)
                if (this -> Components[i] -> Tag == List -> Selected -> SubItems -> Strings[0])
                {
                    exists = true;
                    ((TForm*)this -> Components[i]) -> Show();
                    ((TForm*)this -> Components[i]) -> WindowState = wsNormal;
                }

            if (!exists)
            {
                T = new TTalkForm (this);
                T -> display = List -> Selected -> Caption;
                T -> Caption = T -> display;
                T -> Tag = StrToInt (List -> Selected -> SubItems -> Strings[0]);
                // set icon according to user state
                MainForm->ImageList1->GetIcon(List -> Selected -> ImageIndex, T->Icon);
                T -> Show();
            }
        }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Ustawienia1Click(TObject *Sender)
{
    SettingsForm -> ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Restart1Click(TObject *Sender)
{
    gg_thread -> Terminate();
    Log -> Lines -> Add ("Waiting for GaduThread to terminate...");
    int ret = gg_thread -> WaitFor();
    Log -> Lines -> Add("GaduThread exit, ret. value: " + IntToStr (ret));
    delete gg_thread;

    Log->Lines->Add ("Resetting contacts list...");
    ContactForm -> ResetContactList();

    gg_thread = new TGaduThread(true);
    gg_thread -> Resume();
    Log -> Lines -> Add ("GaduThread restarted");

    TMenuItem* I = gg_thread -> StatusToMenuItem(StrToInt (SettingsForm -> ListView1 -> Items -> Item[2] -> SubItems -> Strings[0]));
    if (I != NULL)
    {
        I -> Checked = true;
        TrayIcon1 -> IconIndex = I -> ImageIndex;
    }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Koniec1Click(TObject *Sender)
{
    if (!bCleanAtExit)
    {
        StatusBar -> Panels -> Items[0] -> Text = "Wychodze...";
        StatusBar -> Repaint();

        gg_thread -> Terminate();
        gg_thread -> WaitFor();

        SettingsForm -> ListView1 -> Items -> Item[5] -> SubItems -> Strings[0] = MainForm -> Left;
        SettingsForm -> ListView1 -> Items -> Item[6] -> SubItems -> Strings[0] = MainForm -> Top;
        SettingsForm -> ListView1 -> Items -> Item[7] -> SubItems -> Strings[0] = MainForm -> Width;
        SettingsForm -> ListView1 -> Items -> Item[8] -> SubItems -> Strings[0] = MainForm -> Height;

        ContactForm -> WriteContacts();
        SettingsForm -> WriteSettings();
        bCleanAtExit = true;
    }
    Application->Terminate();
    Close();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Szukaj1Click(TObject *Sender)
{
    SearchForm -> ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Oczy1Click(TObject *Sender)
{
    Log -> Lines -> Clear();
}
//---------------------------------------------------------------------------

/** Usunicie kontaktu z listy */
void __fastcall TMainForm::Usu1Click(TObject *Sender)
{
    if (List -> Selected != NULL)
        if (MessageBox (Handle, ("Czy napewno chcesz usunac \"" + List -> Selected -> Caption + "\" ?").c_str(), "Potwierdzenie usuniecia", MB_YESNO) == IDYES)
        {
            if (StrToIntDef (List -> Selected -> SubItems -> Strings[0], -1) != -1)	//kontakt posiada identyfikator gg
                gg_thread -> RemoveNotify(StrToInt (List -> Selected -> SubItems -> Strings[0]), StrToInt (List -> Selected -> SubItems -> Strings[7]));
            List -> Items -> Delete (List -> Selected -> Index);
            List -> Selected = NULL;
            Log -> Lines -> Add ("Usunieto.");
        }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WyslijSMS2Click(TObject *Sender)
{
    SMSForm -> Tag = -1;
    SMSForm -> Show();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Pokazdziennik1Click(TObject *Sender)
{
    Log -> Visible = !Log -> Visible;
    Splitter1 -> Visible = Log -> Visible;
    ((TMenuItem*)Sender) -> Checked = Log -> Visible;
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::WyslijSMS1Click(TObject *Sender)
{
    if (List -> Selected && List -> Selected -> SubItems -> Strings[5].Length() >= 9)
    {
        AnsiString phone = List -> Selected -> SubItems -> Strings[5];
        if (phone[1] == '+') phone = phone.SubString (4, phone.Length()-3);
        SMSForm -> Edit1 -> Text = phone.SubString (1,3);
        SMSForm -> Edit2 -> Text = phone.SubString (4,6);
        SMSForm -> Edit3 -> Text = SettingsForm -> ListView1 -> Items -> Item[3] -> SubItems -> Strings[0];
        SMSForm -> Show();
    }
}
//---------------------------------------------------------------------------

/** Wywietlenie hinta nad list kontaktw */
void __fastcall TMainForm::ListInfoTip(TObject *Sender, TListItem *Item,
                                       AnsiString &InfoTip)
{
    InfoTip = Item -> Caption + " (" + Item -> SubItems -> Strings[0] + ")";
    if (Item -> SubItems -> Strings[2] != "") InfoTip += "\n" + Item -> SubItems -> Strings[2] + " " + Item -> SubItems -> Strings[3];
    if (Item -> SubItems -> Strings[1] != "") InfoTip += "\nStatus: " + Item -> SubItems -> Strings[1];
    if (InfoTip[InfoTip.Length()] == 0) InfoTip = InfoTip.SubString (1, InfoTip.Length() - 1);
    InfoTip += "\nP2P  ";
    if (Item -> SubItems -> Strings[9] != "")
    {
        switch (StrToIntDef (Item -> SubItems -> Strings[9], 0))
        {
        case 0:
            InfoTip += "Nie obsuguje";
            break;

        case 1:
            InfoTip += "Maskarada, " + Item -> SubItems -> Strings[8];
            break;

        case 2:
            InfoTip += "Nie ma Ci na licie kontaktw";
            break;

        default:
            InfoTip += "Publiczny: " + Item -> SubItems -> Strings[8] + ":" + Item -> SubItems -> Strings[9];
            break;
        }
    }
    else InfoTip += "Brak danych";
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Edytuj1Click(TObject *Sender)
{
    if (List -> Selected)
    {
        ContactForm -> Show();
        ContactForm -> liEdited = List->Selected;
        ContactForm -> firstname -> Text = List -> Selected -> SubItems -> Strings[2];
        ContactForm -> lastname -> Text = List -> Selected -> SubItems -> Strings[3];
        ContactForm -> nickname -> Text = List -> Selected -> SubItems -> Strings[4];
        ContactForm -> display -> Text = List -> Selected -> Caption;
        ContactForm -> phone -> Text = List -> Selected -> SubItems -> Strings[5];
        ContactForm -> gguin -> Text = List -> Selected -> SubItems -> Strings[0];
        switch (StrToInt (List -> Selected -> SubItems -> Strings[7]))
        {
        case 1:
            ContactForm -> status -> ItemIndex = 0;
            break;

        case 3:
            ContactForm -> status -> ItemIndex = 1;
            break;

        case 4:
            ContactForm -> status -> ItemIndex = 2;
            break;
        }

        ContactForm -> Hide();
        ContactForm -> ShowModal();
    }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Plik2Click(TObject *Sender)
{
    TSaveDialog *D;

    D = new TSaveDialog (NULL);

    D -> DefaultExt = "txt";
    D -> Filter = "Pliki tekstowe (*.txt)|*.txt";
    D -> Options << ofOverwritePrompt;
    if (D -> Execute())
    {
        ContactForm -> WriteContacts (D -> FileName);
        MessageBox (Handle, "Kontakty zostaly zapisane.", "Informacja", MB_OK | MB_ICONINFORMATION);
    }

    delete D;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Plik1Click(TObject *Sender)
{
    TOpenDialog *D;

    D = new TOpenDialog (NULL);

    D -> Filter = "Pliki tekstowe (*.txt)|*.txt";
    if (D -> Execute())
    {
        CopyFile (D -> FileName.c_str(), ContactForm -> contacts_filename.c_str(), false);
        List -> Items -> Clear();
        ContactForm -> ReadContacts();
        Restart1->Click();
        MessageBox (Handle, "Kontakty zostaly zaimportowane.", "Informacja", MB_OK | MB_ICONINFORMATION);
    }

    delete D;
}
//---------------------------------------------------------------------------

/** Eksport kontaktw na serwer GG
*/
void __fastcall TMainForm::SerwerGG2Click(TObject *Sender)
{
    if (gg_thread->dead)
        return;

    AnsiString Contacts = "";
    for (int i = 0; i < List -> Items -> Count; i++)
        Contacts += List -> Items -> Item[i] -> SubItems -> Strings[2] + ";" +
                    List -> Items -> Item[i] -> SubItems -> Strings[3] + ";" +
                    List -> Items -> Item[i] -> SubItems -> Strings[4] + ";" +
                    List -> Items -> Item[i] -> Caption + ";" +
                    List -> Items -> Item[i] -> SubItems -> Strings[5] + ";" +
                    List -> Items -> Item[i] -> SubItems -> Strings[6] + ";" +
                    List -> Items -> Item[i] -> SubItems -> Strings[0] + ";;0;;0;\r\n";
    gg_thread ->ContactExportBuffer = Contacts;
    gg_thread -> UserlistRequest(GG_USERLIST_PUT);
}
//---------------------------------------------------------------------------

/** Import kontaktw z serwera GG. Algorytm:
	- wyczyszczenie istniejcego bufora importu z serwera
	- wysanie dania pobrania listy
	- komunikaty GG_USERLIST_GET_MORE_REPLY uzupeniaj bufor
	- ostatni komunikat GG_USERLIST_GET_MORE_REPLY uzupenia bufor i aktualizuje
	list
*/
void __fastcall TMainForm::SerwerGG1Click(TObject *Sender)
{
    if (gg_thread->dead)
        return;
    gg_thread -> ContactListBuffer = "";
    gg_thread -> UserlistRequest(GG_USERLIST_GET);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Nowe1Click(TObject *Sender)
{
    if (!NewAccntForm)
        NewAccntForm = new TNewAccntForm(this);
    try
    {
        NewAccntForm -> ShowModal();
    }
    __finally
    {
        delete NewAccntForm;
        NewAccntForm = NULL;
    }

}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Skasuj1Click(TObject *Sender)
{
    if (!DelAccntForm)
        DelAccntForm = new TDelAccntForm(this);
    try
    {
        DelAccntForm -> Edit1 -> Text = SettingsForm -> ListView1 -> Items -> Item[0] -> SubItems -> Strings[0];
        DelAccntForm -> Edit2 -> Text = SettingsForm -> ListView1 -> Items -> Item[1] -> SubItems -> Strings[0];
        DelAccntForm -> ShowModal();
    }
    __finally
    {
        delete DelAccntForm;
        DelAccntForm = NULL;
    }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Zmienhaslo1Click(TObject *Sender)
{
    if (!ChPwdForm)
        ChPwdForm = new TChPwdForm(this);
    try
    {
        ChPwdForm -> Edit1 -> Text = SettingsForm -> ListView1 -> Items -> Item[0] -> SubItems -> Strings[0];
        ChPwdForm -> Edit2 -> Text = SettingsForm -> ListView1 -> Items -> Item[1] -> SubItems -> Strings[0];
        ChPwdForm -> ShowModal();
    }
    __finally
    {
        delete ChPwdForm;
        ChPwdForm = NULL;
    }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Przypomnijhaslo1Click(TObject *Sender)
{
    if (!RemindPwdForm)
        RemindPwdForm = new TRemindPwdForm(this);
    try
    {
        RemindPwdForm -> Edit1 -> Text = SettingsForm -> ListView1 -> Items -> Item[0] -> SubItems -> Strings[0];
        RemindPwdForm -> ShowModal();
    }
    __finally
    {
        delete RemindPwdForm;
        RemindPwdForm = NULL;
    }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Opis1Click(TObject *Sender)
{
    if (gg_thread->dead)
        return;
    StatusDescForm -> ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Oprogramie1Click(TObject *Sender)
{
    MessageBox (Handle, "Program rozpowszechniany jest na licencji GPLv2.\n\n\
                Wykorzystano dokumentacj protokou Gadu-Gadu\n\
                z biblioteki libgadu (http://dev.null.pl/ekg)\n\n\
                Program wykorzystuje kod z biblioteki WinImage:\n\
                Copyright  2000, Juan Soulie <jsoulie@cplusplus.com>\n\n\
                 2003 Sadysta (http://sadysta.ath.cx)\n\n\
                 2007 Tomasz Ostrowski (modyfikacje ze rde 1.8.9)\
                ", "O programie...", MB_OK | MB_ICONINFORMATION);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Historia1Click(TObject *Sender)
{
    // History Form wykonuje caFree przy zamykaniu, zwalnianie pamici tutaj jest zbdne
    // bHistoryIsOpen wskazuje, e okno historii jest otwarte
    if(bHistoryIsOpen == false)
    {
        bHistoryIsOpen = true;
        HistoryForm = new THistoryForm(this);
    }
    HistoryForm -> Show();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Mojedane1Click(TObject *Sender)
{
    MeForm -> ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::fonlyClick(TObject *Sender)
{
    int id;
    if(gg_thread->dead)
        return;
    if (status0 -> Checked) id = status0 -> Tag; else
            if (status1 -> Checked) id = status1 -> Tag; else
                    if (status2 -> Checked) id = status2 -> Tag; else
                if (status3 -> Checked) id = status3 -> Tag;

    fonly -> Checked = !fonly -> Checked;
    gg_thread -> ChangeStatus (id, StatusDescForm -> Edit1 -> Text, fonly -> Checked);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormShow(TObject *Sender)
{
    ShowWindow (Application -> Handle, SW_HIDE);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TrayIcon1MouseDown(TObject *Sender,
        TMouseButton Button, TShiftState Shift, int X, int Y)
{
    if (Button == mbLeft || Button == mbMiddle)	///< \todo lewy klawisz widziany jest u mnie jako rodkowy
    {
        Visible = !Visible;
        if (Visible)
        {
            SetActiveWindow (Handle);
            SetForegroundWindow (Handle);
            SetWindowPos (Handle, HWND_TOP, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE);
            Repaint();
        }
    } else
        if (Button == mbRight) PopupMenu1 -> Tag = 1;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ListCompare(TObject *Sender, TListItem *Item1,
                                       TListItem *Item2, int Data, int &Compare)
{
    int i1 = 0, i2 = 0;

    if (Item1 -> SubItems -> Strings[7] == "4") i1 -= 1; else
            if (Item1 -> ImageIndex == 1 || Item1 -> ImageIndex == 2) i1 += 2; else
            if (Item1 -> ImageIndex == 3) i1 += 1;

    if (Item2 -> SubItems -> Strings[7] == "4") i2 -= 1;
    if (Item2 -> ImageIndex == 1 || Item2 -> ImageIndex == 2) i2 += 2; else
        if (Item2 -> ImageIndex == 3) i2 += 1;

    Compare = i2 - i1;
    if (Compare == 0) Compare = CompareText (Item1 -> Caption, Item2 -> Caption);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::KopiujUIN1Click(TObject *Sender)
{
    if (List -> Selected)
        Clipboard() -> SetTextBuf (List -> Selected -> SubItems -> Strings[0].c_str());
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Timer1Timer(TObject *Sender)
{
	static bool once = false;
	if(!once && PlugManager)
	{
		PlugManager->Run();
		if (!PlugManager->PluginCount)
		{
			TMenuItem* SubItem = new TMenuItem( miPlugins );
			SubItem->Caption = "- brak -";
			miPlugins->Add( SubItem );

			SubItem = new TMenuItem( miTrayPlugins );
			SubItem->Caption = "- brak -";
			miTrayPlugins->Add( SubItem );
		}
		else
		{
			for(int i=0; i<PlugManager->PluginCount; i++)
			{
				TMenuItem* SubItem = new TMenuItem( miPlugins );
				SubItem->Caption = PlugManager->PluginList[i].Name();
				SubItem->Tag = i;
				SubItem->OnClick = miPluginsClick;
				miPlugins->Add( SubItem );

				SubItem = new TMenuItem( miTrayPlugins );
				SubItem->Caption = PlugManager->PluginList[i].Name();
				SubItem->Tag = i;
				SubItem->OnClick = miPluginsClick;
				miTrayPlugins->Add( SubItem );
			}
		}
		once = true;
	}
	if (gg_thread != NULL && gg_thread -> dead &&
		SettingsForm -> CheckBox4 -> Checked &&
		!status3 -> Checked &&
		!gg_thread->bDisconnectedByServer)
	{
		Log -> Lines -> Add ("[GaduThread] Auto-restart feature active");
		Restart1 -> Click();
	}
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ToolButton4Click(TObject *Sender)
{
    Szukaj1 -> Click();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ToolButton2Click(TObject *Sender)
{
    Ustawienia1 -> Click();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ToolButton3Click(TObject *Sender)
{
    Opis1 -> Click();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::PopupMenu1Popup(TObject *Sender)
{
    if (((TPopupMenu*)Sender) -> Tag == 1)
    {
        Koniec2 -> Visible = true;
        ((TPopupMenu*)Sender) -> Tag = 0;
    } else Koniec2 -> Visible = false;
}

//---------------------------------------------------------------------------
void __fastcall TMainForm::Dziennikaktywnosci1Click(TObject *Sender)
{
    ActivityForm -> Show();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ToolButton5Click(TObject *Sender)
{
    ActivityForm -> Show();
}
//---------------------------------------------------------------------------

TListItem* TMainForm::FindUIN (int UIN)
{
    for (int i = 0; i < List -> Items -> Count; i++)
        if (StrToIntDef (List -> Items -> Item[i] -> SubItems -> Strings[0], -1) == UIN)
            return List -> Items -> Item[i];
    return NULL;
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::FormCloseQuery(TObject *Sender, bool &CanClose)
{
    // This application closes to the system tray. When they push
    // close, hide the app, but don't let the program close.
    if(!Application->Terminated)
    {
        //ShowWindow(Application->Handle, SW_HIDE);
        Visible = false;
        CanClose = false;
        Action = caNone;
        return;
    }

    /*
    The OnShow is always called when the app comes up but you see that the
    different ways of closing cause a different sequence of events to be called.
    The most pathological case is the last one, here the form is not properly
    destroyed at all! And there is no event that is called for each of these
    cases. So you have to put the cleanup code into two locations, OnCloseQuery
    and OnDestroy, and guard against duplicate execution.
    OnCloseQuery has a special problem: it may be called more than once! If it
    is called because Windows goes down, all apps will be asked to comply, and
    if
    another app refuses your app may _not_ go down after you have already done
    your cleanup duty. You can prevent this by calling Application.Terminate
    after setting CanClose to True.
    */

    // gg_thread -> ChangeStatus (4, StatusDescForm -> Edit1 -> Text, fonly -> Checked);

    if(!bCleanAtExit)
    {
        StatusBar -> Panels -> Items[0] -> Text = "Zamykam...";
        StatusBar -> Repaint();

        gg_thread -> Terminate();
        gg_thread -> WaitFor();

        SettingsForm -> ListView1 -> Items -> Item[5] -> SubItems -> Strings[0] = MainForm -> Left;
        SettingsForm -> ListView1 -> Items -> Item[6] -> SubItems -> Strings[0] = MainForm -> Top;
        SettingsForm -> ListView1 -> Items -> Item[7] -> SubItems -> Strings[0] = MainForm -> Width;
        SettingsForm -> ListView1 -> Items -> Item[8] -> SubItems -> Strings[0] = MainForm -> Height;

        ContactForm -> WriteContacts();
        SettingsForm -> WriteSettings();
        bCleanAtExit = true;
    }
    // Action = caNone;
    CanClose = true;
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::WMQuery(TWMQueryEndSession &Msg)
{
    Msg.Result = 1;
    if(!bCleanAtExit)
    {
        StatusBar -> Panels -> Items[0] -> Text = "Zamykam...";
        StatusBar -> Repaint();

        gg_thread -> Terminate();
        gg_thread -> WaitFor();

        SettingsForm -> ListView1 -> Items -> Item[5] -> SubItems -> Strings[0] = MainForm -> Left;
        SettingsForm -> ListView1 -> Items -> Item[6] -> SubItems -> Strings[0] = MainForm -> Top;
        SettingsForm -> ListView1 -> Items -> Item[7] -> SubItems -> Strings[0] = MainForm -> Width;
        SettingsForm -> ListView1 -> Items -> Item[8] -> SubItems -> Strings[0] = MainForm -> Height;

        ContactForm -> WriteContacts();
        SettingsForm -> WriteSettings();
        bCleanAtExit = true;
    }
    Application->Terminate();
}

void __fastcall TMainForm::ListDrawItem(TCustomListView *Sender,
                                        TListItem *Item, TRect &ARect, TOwnerDrawState State)
{
    if(Item->SubItems->Count < 5)
        return;	//nowo dodawany rekord - nie rysuj jeszcze
    TCanvas *C = Sender -> Canvas;
    TSize e;
    int offset = 0;
    if (State.Contains (odSelected))
    {
        C -> Font -> Color = clBlack;
        C -> Brush -> Color = (TColor)0xCCCCCC;
    } else
    {
        C -> Font -> Color = clBlack;
        C -> Brush -> Color = (TColor)0xEEEEEE;
    }

    C->Font->Name = "Arial";
    C -> Font ->Height = 14;

    e = C -> TextExtent (Item -> Caption);	//Returns the width and height, in pixels, of a string rendered in the current font
    //		if (State.Contains (odSelected))
    //			C->Brush->Color = clGray;
    //			//C -> Rectangle (25, Item -> Position.y, e.cx + 30, Item -> Position.y + 16);
    //		else
    //            C->Brush->Color = clWhite;
    //			//C -> FillRect (Rect (25, Item -> Position.y, e.cx + 30, Item -> Position.y + 16));

    //narysowanie ikony stanu kontaktu
    ImageList5 -> Draw (C, 5, Item -> Position.y - 5, Item -> ImageIndex, true);	//void Draw (TCanvas * Canvas, int X, int Y, int Index, bool Enabled);

    C -> TextOut (26, Item -> Position.y, Item -> Caption);
    C -> Font ->Height = 12;
    C ->Font->Color = clGreen;
    C -> TextOut (26, Item -> Position.y + 15, Item -> SubItems -> Strings[1]);

    //        if (Item -> SubItems -> Strings[1] != "")
    //        {
    //				// status opisowy - ikona "dymek"
    //				ImageList3 -> Draw (C, 31 + e.cx, Item -> Position.y, Item -> StateIndex, true);
    //				offset += 16;
    //		}
    if (Item -> SubItems -> Strings[5] != "")
    {
        // ikona telefonu
        ImageList3 -> Draw (C, offset + 31 + e.cx, Item -> Position.y, 1, true);
        offset += 16;
    }
    if (Item -> SubItems -> Strings[7] != "1")
    {
        //blokada/friend (?)
        if (Item -> SubItems -> Strings[7] == "3")
            ImageList3 -> Draw (C, offset + 31 + e.cx, Item -> Position.y, 2, true); else
            if (Item -> SubItems -> Strings[7] == "4")
                ImageList3 -> Draw (C, offset + 31 + e.cx, Item -> Position.y, 3, true);
        offset += 16;
    }
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::miPluginsClick(TObject *Sender)
{
	PlugManager->ShowPlugin(((TMenuItem*)Sender)->Tag);
}
//---------------------------------------------------------------------------

